@@ -25,6 +25,7 @@ int gengetopt_define_package (char * s) ;
 int gengetopt_define_version (char * s) ;
 int gengetopt_define_purpose (char * s) ;
 int gengetopt_add_option (char * long_opt, char short_opt, char * desc, 
-			  int type, int flagstat, int required);
+			  int type, int flagstat, int required,
+                          char *default_value);
 
 #endif /* _GENGETOPT_H */
