@@ -11,7 +11,8 @@ This file is licensed to you under the license specified in the included file
 #include "config.h"
 #endif
 
-struct gengetopt_option {
+struct gengetopt_option
+{
   char short_opt;
   char * long_opt;
   char * desc;
@@ -20,6 +21,9 @@ struct gengetopt_option {
   int required;
   struct gengetopt_option * next;
   char * var_arg; /* canonized long_opt + "_arg" = argument var */
+  int default_given ; /* if a default is given */
+  char * default_string ; /* default value for this option, if string */
+  double default_num ; /* default value for this option, if numeric */
 };
 
 #endif /* _GENGETOPT_GGOS_H */
