@@ -46,6 +46,7 @@ static void dec_indent() ;
 static void check_option_given( char *var_arg,
                                 char *long_opt, char short_opt ) ;
 static void generate_error_handle();
+static char *canonize_names(char * name);
 
 #define ARGS_STRUCT "args_info"
 #define ARGS_STRUCT_NAME PACKAGE "_" ARGS_STRUCT
@@ -105,7 +106,7 @@ do_check_option_given (struct gengetopt_option *opt)
 	case ARG_STRING:
         check_option_given (opt->var_arg, opt->long_opt, opt->short_opt);
         indent ();
-        printf ("%s->%s_arg = gengetopt_strdup (optarg);\n",
+        printf ("%s->%s_arg = strdup (optarg);\n",
 				ARGS_STRUCT, opt->var_arg);
         break;
 	case ARG_INT:
@@ -176,9 +177,12 @@ generate_cmdline_parser (char *function_name, short unamed_options,
 
   FILE *output_file ;
 
+  char *filename_canonized = 0;
+
   char *header_filename ;
   char *c_filename ;
-  parser_function_name = function_name; /* initialize static data */
+  parser_function_name = canonize_names (function_name); 
+  /* initialize static data */
 
   header_filename = create_filename (filename, header_ext) ;
   c_filename = create_filename (filename, c_ext) ;
@@ -205,8 +209,10 @@ generate_cmdline_parser (char *function_name, short unamed_options,
   printf ("/* %s */\n\n", header_filename);
   printf ("/* File autogenerated by gengetopt version %s  */\n\n",
           VERSION) ;
-  printf ("#ifndef _%s_%s\n", filename, header_ext);
-  printf ("#define _%s_%s\n\n", filename, header_ext);
+
+  filename_canonized = canonize_names (filename);
+  printf ("#ifndef _%s_%s\n", filename_canonized, header_ext);
+  printf ("#define _%s_%s\n\n", filename_canonized, header_ext);
 
   printf ("#ifdef __cplusplus\n");
   printf ("extern \"C\" {\n");
@@ -273,6 +279,15 @@ generate_cmdline_parser (char *function_name, short unamed_options,
  			printf ("%s_arg", opt->var_arg);
 
 		printf (";\t/* %s", opt->desc);
+         
+                if (opt->default_given)
+                  {
+                    if (opt->type == ARG_STRING)
+                      printf (" (default='%s')", opt->default_string);
+                    else
+                      printf (" (default=%.0f)", opt->default_num);
+                  }
+
 		if (opt->type == ARG_FLAG)
                   {
                     if (opt->flagstat)
@@ -280,6 +295,7 @@ generate_cmdline_parser (char *function_name, short unamed_options,
                     else    
                       printf (" (default=off)");
                   }
+
 		printf (".  */\n");
 	}
 
@@ -325,16 +341,18 @@ generate_cmdline_parser (char *function_name, short unamed_options,
   printf ("} ;\n\n" ) ;
 
   printf ("int %s (int argc, char * const *argv, struct %s *%s);\n\n", 
-          function_name, ARGS_STRUCT_NAME, ARGS_STRUCT);
+          parser_function_name, ARGS_STRUCT_NAME, ARGS_STRUCT);
 
-  printf ("void %s_print_help(void);\n", function_name);
-  printf ("void %s_print_version(void);\n\n", function_name);
+  printf ("void %s_print_help(void);\n", parser_function_name);
+  printf ("void %s_print_version(void);\n\n", parser_function_name);
 
   printf ("#ifdef __cplusplus\n");
   printf ("}\n");
   printf ("#endif /* __cplusplus */\n");
 
-  printf ("#endif /* _%s_%s */\n", filename, header_ext);
+  printf ("#endif /* _%s_%s */\n", filename_canonized, header_ext);
+
+  free (filename_canonized); // it's no longer useful
 
   fclose (output_file) ;
 
@@ -380,6 +398,9 @@ generate_cmdline_parser (char *function_name, short unamed_options,
 "#include <getopt.h>\n"
 "#endif\n"
 "\n"
+"#ifndef HAVE_STRDUP\n"
+"#define strdup gengetopt_strdup\n"
+"#endif /* HAVE_STRDUP */\n\n"
 "#include \"%s\"\n"
 "\n",
 header_filename);
@@ -394,7 +415,8 @@ header_filename);
 "void\n"
 "%s_print_help (void)\n"
 "{\n"
-"  %s_print_version ();\n", function_name, function_name, function_name);
+"  %s_print_version ();\n", 
+          parser_function_name, parser_function_name, parser_function_name);
   printf ("  printf(\"\\n\"\n");
   if (gengetopt_purpose != NULL) {
     char *ptr;
@@ -538,7 +560,15 @@ header_filename);
       for (w = 3+strlen(opt->long_opt)+
 	     strlen(arg_names[opt->type]); w < max_long; w++)
 	printf (" ");
-      printf ("  %s\\n\");\n", opt->desc);
+      printf ("  %s", opt->desc);
+      if (opt->default_given)
+        {
+          if (opt->type == ARG_STRING)
+            printf (" (default='%s')", opt->default_string);
+          else
+            printf (" (default=%.0f)", opt->default_num);
+        }
+      printf ("\\n\");\n");
     }
   }
   printf ("}\n\n\n");
@@ -565,7 +595,7 @@ header_filename);
 "{\n"
 "  int c;\t/* Character of the parsed option.  */\n"
 "  int missing_required_options = 0;\t\n"
-"\n", function_name, ARGS_STRUCT_NAME, ARGS_STRUCT);
+"\n", parser_function_name, ARGS_STRUCT_NAME, ARGS_STRUCT);
 
 inc_indent ();
 
@@ -584,8 +614,12 @@ inc_indent ();
 	if (opt->type == ARG_STRING)
           {
             indent ();
-	    printf ("%s->%s_arg = NULL; \\\n",
-			ARGS_STRUCT, opt->var_arg);
+            if (opt->default_given)
+              printf ("%s->%s_arg = strdup(\"%s\") ;\\\n",
+                      ARGS_STRUCT, opt->var_arg, opt->default_string);
+            else
+              printf ("%s->%s_arg = NULL; \\\n",
+                      ARGS_STRUCT, opt->var_arg);
           }
         else if (opt->type == ARG_FLAG)
           {
@@ -593,6 +627,12 @@ inc_indent ();
             printf ("%s->%s_flag", ARGS_STRUCT, opt->var_arg);
             printf (" = %d;\\\n", opt->flagstat);
           }
+        else if (opt->type != ARG_NO && opt->default_given)
+          {
+            indent ();
+            printf ("%s->%s_arg", ARGS_STRUCT, opt->var_arg);
+            printf (" = %.0f;\\\n", opt->default_num);
+          }
 
   printf ("}\n\n") ;
 
@@ -684,9 +724,9 @@ inc_indent ();
       
       printf ("clear_args ();\n");
       indent ();
-      printf ("%s_print_help ();\n", function_name);
+      printf ("%s_print_help ();\n", parser_function_name);
       indent ();
-      printf ("exit (0);\n");
+      printf ("exit (EXIT_SUCCESS);\n");
 
       dec_indent ();
     }
@@ -700,9 +740,9 @@ inc_indent ();
       indent ();
       printf ("clear_args ();\n");
       indent ();
-      printf ("%s_print_version ();\n", function_name);
+      printf ("%s_print_version ();\n", parser_function_name);
       indent ();
-      printf ("exit (0);\n");
+      printf ("exit (EXIT_SUCCESS);\n");
       dec_indent ();
     }
 
@@ -804,9 +844,9 @@ inc_indent ();
         inc_indent ();
         indent ();
         if (opt->short_opt)
-            printf ("fprintf (stderr, \"%%s: `--%s' (`-%c') option required!\\n\", PACKAGE);\n", opt->long_opt, opt->short_opt);
+            printf ("fprintf (stderr, \"%%s: `--%s' (`-%c') option required\\n\", PACKAGE);\n", opt->long_opt, opt->short_opt);
         else
-            printf ("fprintf (stderr, \"%%s: `--%s' option required!\\n\", PACKAGE);\n", opt->long_opt);
+            printf ("fprintf (stderr, \"%%s: `--%s' option required\\n\", PACKAGE);\n", opt->long_opt);
         indent ();
         printf ("missing_required_options = 1;\n");
         dec_indent ();
@@ -850,7 +890,7 @@ inc_indent ();
 
       inc_indent ();
       indent ();
-      printf ("%s->inputs[ i++ ] = gengetopt_strdup (argv[optind++]) ; \n", 
+      printf ("%s->inputs[ i++ ] = strdup (argv[optind++]) ; \n", 
               ARGS_STRUCT);
 
       dec_indent ();
@@ -918,7 +958,25 @@ void
 generate_error_handle()
 {
   if (handle_error)
-    printf ("exit (1);\n");
+    printf ("exit (EXIT_FAILURE);\n");
   else
-    printf ("return (1);\n");
+    printf ("return (EXIT_FAILURE);\n");
+}
+
+/*
+  return a copy of the string passed after canonizing it (i.e. '-' and
+  '.' are transformed in '_'.
+*/
+char *
+canonize_names (char *name)
+{
+  char *pvar;
+  char *p;
+
+  pvar = strdup (name);
+
+  for (p = pvar; *p; ++p)
+    if (*p == '.' || *p == '-') *p = '_';
+
+  return pvar;
 }
