@@ -14,6 +14,6 @@ extern int gengetopt_count_line;
 void
 yyerror (char *s)
 {
-  fprintf (stderr, "gengetopt: line %d: %s\n", gengetopt_count_line, s);
+  fprintf (stderr, "gengetopt: %d: %s\n", gengetopt_count_line, s);
 }
 
